#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import sys
import string
from math import *
import traceback

import QUB
import QUBB
import Reports
import qubtree

import win32com.client
from pythoncom import CoInitialize, CoUninitialize



CoInitialize()


class ExcelReport:
    def __init__(self):
        self.excel = win32com.client.Dispatch("Excel.Application")
        self.book = self.excel.Workbooks.Add(-4167) # XLWBatWorksheet
        self.sheet = self.book.Worksheets.Item(1)
        self.sheet.Cells.HorizontalAlignment = -4108 # 0xffffeff4 # xlHAlignCenter
        self.excel.Visible = 1

    def __del__(self):
        del self.sheet
        del self.book
        del self.excel

class ExcelColReport(ExcelReport):
    def __init__(self):
        ExcelReport.__init__(self)
        self.colCount = 1

    # dynamically find or add column header each time so user can rearrange columns
    def findRow(self, name):
        def rowgen(Cells):
            i = 2
            while Cells(i, 1).Value:
                yield(i, Cells(i, 1).Value)
                i += 1
        last = 1
        for i, rowname in rowgen(self.sheet.Cells):
            if rowname == name:
                return i
            last = i
        # not found:
        i = last + 1
        self.sheet.Cells(i, 1).Value = name
        return i
    
    def writeCol(self, lbl, enum):# of (name, value)
        cells = self.sheet.Cells
        self.colCount += 1
        cells(1, self.colCount).Value = lbl
        for name, value in enum:
            cells(self.findRow(name), self.colCount).Value = str(value)

    def test(self):
        self.sheet.Cells(1,1).Value = ''
        
class ExcelRowReport(ExcelReport):
    def __init__(self):
        ExcelReport.__init__(self)
        self.rowCount = 1

    # dynamically find or add column header each time so user can rearrange columns
    def findCol(self, name):
        def colgen(Cells):
            i = 2
            while Cells(1, i).Value:
                yield(i, Cells(1, i).Value)
                i += 1
        last = 1
        for i, colname in colgen(self.sheet.Cells):
            if colname == name:
                return i
            last = i
        # not found:
        i = last + 1
        self.sheet.Cells(1, i).Value = name
        return i
    
    def writeRow(self, lbl, enum):# of (name, value)
        cells = self.sheet.Cells
        self.rowCount += 1
        cells(self.rowCount, 1).Value = lbl
        for name, value in enum:
            cells(self.rowCount, self.findCol(name)).Value = str(value)

    def test(self):
        self.sheet.Cells(1,1).Value = ''
        
def EnumerateValues(node, Accept):
    child = node.child
    while not child.isNull:
        data = child.data
        if Accept(child):
            if data.type == qubtree.QTR_TYPE_STRING:
                yield(child.name, str(data))
            elif len(data) == 1:
                yield(child.name, data[0])
            else:
                for i in range(len(data)):
                    yield(("%s %d" % (child.name, (i+1))), data[i])
        child = child.sibling

#these are the criteria from QUB_QVO_QVR:  TQUB_Results_SumPanel
def AcceptLikeQVR(node):
    data = node.data
    return (len(data) > 0) and (len(data) < 200) and (node.child.sibling.isNull) and (node.name <> 'Hold')

activeColReports = {}
def EnsureExcelColReport(results, context):
    rtype = str(results.node.find('ResultType').data)
    if rtype in activeColReports.keys():
        try:
            activeColReports[rtype].test()
        except:
            del activeColReports[rtype]
    if rtype in activeColReports.keys():
        context.report = activeColReports[rtype]
    else:
        try:
            win32com.client.gencache.EnsureModule('{00020813-0000-0000-C000-000000000046}',0,1,3)
        except:
            traceback.print_exc()
        context.report = ExcelColReport()
        activeColReports[rtype] = context.report
    return context.report

def ReportExcelCol(results, context):
    rep = EnsureExcelColReport(results, context)
    lbl = str(results.node['Date'].data)
    rep.writeCol(lbl, EnumerateValues(results.node, AcceptLikeQVR))

Reports.Add('Excel Append Col', ReportExcelCol, ['Mac'])

activeRowReports = {}
def EnsureExcelRowReport(results, context):
    rtype = str(results.node.find('ResultType').data)
    if rtype in activeRowReports.keys():
        try:
            activeRowReports[rtype].test()
        except:
            del activeRowReports[rtype]
    if rtype in activeRowReports.keys():
        context.report = activeRowReports[rtype]
    else:
        try:
            win32com.client.gencache.EnsureModule('{00020813-0000-0000-C000-000000000046}',0,1,3)
        except:
            traceback.print_exc()
        context.report = ExcelRowReport()
        activeRowReports[rtype] = context.report
    return context.report

def ReportExcelRow(results, context):
    rep = EnsureExcelRowReport(results, context)
    lbl = str(results.node['Date'].data)
    rep.writeRow(lbl, EnumerateValues(results.node, AcceptLikeQVR))

Reports.Add('Excel Append Row', ReportExcelRow, ['Mac'])
